@@ -17,6 +17,7 @@
 """Tests for functions in helpers."""
 
 import io
+import json
 import os
 import sys
 import tempfile
@@ -24,6 +25,7 @@ import unittest
 
 from unittest.mock import patch
 from urllib.error import HTTPError
+from urllib.request import Request
 
 import logassert
 
@@ -131,9 +133,13 @@ class GetInterpreterInfoTestCase(unittest.TestCase):
 
     def test_requested_not_exists(self):
         side_effect = IOError("[Errno 2] No such file or directory: 'pythonME'")
-        with patch.object(helpers, 'logged_exec',
-                          side_effect=side_effect), self.assertRaises(SystemExit):
-            helpers._get_interpreter_info('pythonME')
+
+        with patch('fades.helpers.logged_exec') as mock_lexec:
+            mock_lexec.side_effect = side_effect
+
+            with self.assertRaises(Exception):
+                helpers._get_interpreter_info('pythonME')
+
         self.assertLoggedError("Error getting requested interpreter version:"
                                " [Errno 2] No such file or directory: 'pythonME'")
 
@@ -150,7 +156,7 @@ class GetLatestVersionNumberTestCase(unittest.TestCase):
                 mock_urlopen.return_value = fh
                 last_version = helpers.get_latest_version_number("some_package")
         mock_urlopen.assert_called_once_with(helpers.BASE_PYPI_URL.format(name="some_package"))
-        self.assertEquals(last_version, '2.8.1')
+        self.assertEqual(last_version, '2.8.1')
 
     def test_get_version_wrong(self):
         with patch('urllib.request.urlopen') as mock_urlopen:
@@ -191,8 +197,8 @@ class CheckPyPIUpdatesTestCase(unittest.TestCase):
                 dep_django = dependencies['pypi'][0]
                 dep_request = dependencies['pypi'][1]
                 self.assertLoggedInfo('There is a new version of django: 1.9')
-                self.assertEquals(dep_request.specs, [('==', '2.1')])
-                self.assertEquals(dep_django.specs, [('==', '1.7.5')])
+                self.assertEqual(dep_request.specs, [('==', '2.1')])
+                self.assertEqual(dep_django.specs, [('==', '1.7.5')])
                 self.assertLoggedInfo("The latest version of 'requests' is 2.1 and will use it.")
 
     def test_check_pypi_updates_with_a_higher_version_of_a_package(self):
@@ -342,28 +348,22 @@ class CheckPackageExistenceTestCase(unittest.TestCase):
     def test_error_hitting_pypi(self):
         dependency = parsing.parse_manual(["foo"])
 
-        with patch('sys.exit') as mocked_exit:
+        with self.assertRaises(Exception):
             with patch('urllib.request.urlopen') as mock_urlopen:
                 mock_urlopen.side_effect = ValueError("cabum!!")
 
                 helpers.check_pypi_exists(dependency)
 
-        self.assertTrue(mocked_exit.called)
-        mocked_exit.assert_called_once_with(1)
-
     def test_status_code_error(self):
         dependency = parsing.parse_manual(["foo"])
 
-        with patch('sys.exit') as mocked_exit:
+        with self.assertRaises(Exception):
             with patch('urllib.request.urlopen') as mock_urlopen:
                 mock_http_error = HTTPError("url", 400, "mgs", {}, io.BytesIO())
                 mock_urlopen.side_effect = mock_http_error
 
                 helpers.check_pypi_exists(dependency)
 
-        self.assertTrue(mocked_exit.called)
-        mocked_exit.assert_called_once_with(1)
-
     def test_redirect_response(self):
         deps = parsing.parse_manual(["foo"])
 
@@ -375,3 +375,152 @@ class CheckPackageExistenceTestCase(unittest.TestCase):
                 exists = helpers.check_pypi_exists(deps)
         self.assertTrue(exists)
         self.assertLoggedWarning("Got a (unexpected) HTTP_STATUS")
+
+
+class ScriptDownloaderTestCase(unittest.TestCase):
+    """Check the script downloader."""
+
+    def setUp(self):
+        logassert.setup(self, 'fades.helpers')
+
+    def test_external_public_function(self):
+        test_url = "http://scripts.com/foobar.py"
+        test_content = "test content of the remote script ññ"
+        with patch('fades.helpers._ScriptDownloader') as mock_downloader_class:
+            mock_downloader = mock_downloader_class()
+            mock_downloader.get.return_value = test_content
+            mock_downloader.name = 'mock downloader'
+            filepath = helpers.download_remote_script(test_url)
+
+        # plan to remove the downloaded content (so test remains clean)
+        self.addCleanup(os.unlink, filepath)
+
+        # checks
+        mock_downloader_class.assert_called_with(test_url)
+        self.assertLoggedInfo(
+            "Downloading remote script", test_url, repr(filepath), 'mock downloader')
+        with open(filepath, "rt", encoding='utf8') as fh:
+            self.assertEqual(fh.read(), test_content)
+
+    def test_decide_linkode(self):
+        url = "http://linkode.org/#02c5nESQBLEjgBRhUwJK74"
+        downloader = helpers._ScriptDownloader(url)
+        name = downloader._decide()
+        self.assertEqual(name, 'linkode')
+
+    def test_decide_pastebin(self):
+        url = "https://pastebin.com/sZGwz7SL"
+        downloader = helpers._ScriptDownloader(url)
+        name = downloader._decide()
+        self.assertEqual(name, 'pastebin')
+
+    def test_decide_gist(self):
+        url = "https://gist.github.com/facundobatista/6ff4f75760a9acc35e68bae8c1d7da1c"
+        downloader = helpers._ScriptDownloader(url)
+        name = downloader._decide()
+        self.assertEqual(name, 'gist')
+
+    def test_downloader_raw(self):
+        test_url = "http://scripts.com/foobar.py"
+        raw_service_response = b"test content of the remote script"
+        downloader = helpers._ScriptDownloader(test_url)
+        with patch('urllib.request.urlopen') as mock_urlopen:
+            with patch('http.client.HTTPResponse') as mock_http_response:
+                mock_http_response.read.return_value = raw_service_response
+                mock_urlopen.return_value = mock_http_response
+
+            content = downloader.get()
+
+        # check urlopen was called with the proper url, and passing correct headers
+        headers = {
+            'Accept': 'text/plain',
+            'User-agent': helpers._ScriptDownloader.USER_AGENT,
+        }
+        (call,) = mock_urlopen.mock_calls
+        (called_request,) = call[1]
+        self.assertIsInstance(called_request, Request)
+        self.assertEqual(called_request.full_url, test_url)
+        self.assertEqual(called_request.headers, headers)
+        self.assertEqual(content, raw_service_response.decode("utf8"))
+
+    def test_downloader_linkode(self):
+        test_url = "http://linkode.org/#02c5nESQBLEjgBRhUwJK74"
+        test_content = "test content of the remote script áéíóú"
+        raw_service_response = json.dumps({
+            'content': test_content,
+            'morestuff': 'whocares',
+        }).encode("utf8")
+
+        downloader = helpers._ScriptDownloader(test_url)
+        with patch('urllib.request.urlopen') as mock_urlopen:
+            with patch('http.client.HTTPResponse') as mock_http_response:
+                mock_http_response.read.return_value = raw_service_response
+                mock_urlopen.return_value = mock_http_response
+
+            content = downloader.get()
+
+        # check urlopen was called with the proper url, and passing correct headers
+        headers = {
+            'Accept': 'application/json',
+            'User-agent': helpers._ScriptDownloader.USER_AGENT,
+        }
+        (call,) = mock_urlopen.mock_calls
+        (called_request,) = call[1]
+        self.assertIsInstance(called_request, Request)
+        self.assertEqual(
+            called_request.full_url, "https://linkode.org/api/1/linkodes/02c5nESQBLEjgBRhUwJK74")
+        self.assertEqual(called_request.headers, headers)
+        self.assertEqual(content, test_content)
+
+    def test_downloader_pastebin(self):
+        test_url = "http://pastebin.com/sZGwz7SL"
+        test_content = "test content of the remote script áéíóú"
+        raw_service_response = test_content.encode("utf8")
+
+        downloader = helpers._ScriptDownloader(test_url)
+        with patch('urllib.request.urlopen') as mock_urlopen:
+            with patch('http.client.HTTPResponse') as mock_http_response:
+                mock_http_response.read.return_value = raw_service_response
+                mock_urlopen.return_value = mock_http_response
+
+            content = downloader.get()
+
+        # check urlopen was called with the proper url, and passing correct headers
+        headers = {
+            'Accept': 'text/plain',
+            'User-agent': helpers._ScriptDownloader.USER_AGENT,
+        }
+        (call,) = mock_urlopen.mock_calls
+        (called_request,) = call[1]
+        self.assertIsInstance(called_request, Request)
+        self.assertEqual(
+            called_request.full_url, "https://pastebin.com/raw/sZGwz7SL")
+        self.assertEqual(called_request.headers, headers)
+        self.assertEqual(content, test_content)
+
+    def test_downloader_gist(self):
+        test_url = "http://gist.github.com/facundobatista/6ff4f75760a9acc35e68bae8c1d7da1c"
+        test_content = "test content of the remote script áéíóú"
+        raw_service_response = test_content.encode("utf8")
+
+        downloader = helpers._ScriptDownloader(test_url)
+        with patch('urllib.request.urlopen') as mock_urlopen:
+            with patch('http.client.HTTPResponse') as mock_http_response:
+                mock_http_response.read.return_value = raw_service_response
+                mock_urlopen.return_value = mock_http_response
+
+            content = downloader.get()
+
+        # check urlopen was called with the proper url, and passing correct headers
+        headers = {
+            'Accept': 'text/plain',
+            'User-agent': helpers._ScriptDownloader.USER_AGENT,
+        }
+        (call,) = mock_urlopen.mock_calls
+        (called_request,) = call[1]
+        self.assertIsInstance(called_request, Request)
+        self.assertEqual(
+            called_request.full_url,
+            "https://gist.github.com/facundobatista/6ff4f75760a9acc35e68bae8c1d7da1c/raw")
+        self.assertEqual(called_request.headers, headers)
+        self.assertEqual(content, test_content)
