@@ -18,12 +18,14 @@
 
 import io
 import unittest
+import os
 
 import logassert
 
 from pkg_resources import parse_requirements
 
 from fades import parsing, REPO_PYPI, REPO_VCS
+from tests import create_tempfile
 
 
 def get_req(text):
@@ -292,8 +294,8 @@ class FileParsingTestCase(unittest.TestCase):
         parsed = parsing._parse_content(io.StringIO("""
             from foo bar import :(   # fades
         """))
-        self.assertLoggedWarning("Not understood import info",
-                                 "['from', 'foo', 'bar', 'import', ':(']")
+        self.assertLoggedDebug("Not understood import info",
+                               "['from', 'foo', 'bar', 'import', ':(']")
         self.assertDictEqual(parsed, {})
 
     def test_strange_fadesinfo(self):
@@ -452,6 +454,30 @@ class FileParsingTestCase(unittest.TestCase):
             REPO_PYPI: [get_req('bar')],
         })
 
+    def test_fades_and_hashtag_mentioned_in_code(self):
+        """Test the case where a string contains both: fades and hashtag (#)
+        but is not an import.
+        """
+        parsed = parsing._parse_content(io.StringIO("""
+          'http://fades.readthedocs.io/en/release-7-0/readme.html#how-to-use-it'
+        """))
+        self.assertDictEqual(parsed, {})
+
+    def test_fades_and_hashtag_mentioned_in_code_mixed_with_imports(self):
+        parsed = parsing._parse_content(io.StringIO("""import requests  # fades
+
+          'http://fades.readthedocs.io/en/release-7-0/readme.html#how-to-use-it'
+        """))
+        self.assertDictEqual(parsed, {
+            REPO_PYPI: [get_req('requests')]
+        })
+
+    def test_fades_user_strange_comment_with_hashtag_ignored(self):
+        parsed = parsing._parse_content(io.StringIO("""
+          import foo # fades==2 # Some comment with #hashtash
+        """))  # noqa
+        self.assertDictEqual(parsed, {})
+
 
 class ManualParsingTestCase(unittest.TestCase):
     """Check the manual parsing."""
@@ -729,3 +755,63 @@ class VCSDependencyTestCase(unittest.TestCase):
         self.assertTrue(dep1 != dep3)
         self.assertFalse(dep1 == 123)
         self.assertFalse(dep1 == "testurl")
+
+
+class FileReqsParsingTestCase(unittest.TestCase):
+    """Check the requirements parsing from a reqs.txt file."""
+
+    def setUp(self):
+        logassert.setup(self, 'fades.parsing')
+
+    def test_requirement_files(self):
+        requirement_file = create_tempfile(self, ['foo'])
+        parsed = parsing.parse_reqfile(requirement_file)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [get_req('foo')]})
+
+    def test_nested_requirement_files(self):
+        requirement_file = create_tempfile(self, ['foo'])
+        requirement_file_nested = create_tempfile(
+            self, ['bar\n-r {}'.format(requirement_file)])
+        parsed = parsing.parse_reqfile(requirement_file_nested)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [get_req('bar'),
+                                                  get_req('foo')]})
+
+    def test_nested_requirement_files_invalid_format(self):
+        requirement_file_nested = create_tempfile(self, ['foo\n-r'])
+        parsed = parsing.parse_reqfile(requirement_file_nested)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [get_req('foo')]})
+        self.assertLoggedWarning(
+            "Invalid format to indicate a nested requirements file:")
+
+    def test_nested_requirement_files_not_pwd(self):
+        requirement_file = create_tempfile(self, ['foo'])
+        fname = os.path.basename(requirement_file)
+        requirement_file_nested = create_tempfile(
+            self, ['bar\n-r {}'.format(fname)])
+        parsed = parsing.parse_reqfile(requirement_file_nested)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [get_req('bar'),
+                                                  get_req('foo')]})
+
+    def test_nested_requirement_files_first_line(self):
+        requirement_file = create_tempfile(self, ['foo'])
+        requirement_file_nested = create_tempfile(
+            self, ['\n-r {}\nbar'.format(requirement_file)])
+        parsed = parsing.parse_reqfile(requirement_file_nested)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [get_req('foo'),
+                                                  get_req('bar')]})
+
+    def test_two_nested_requirement_files(self):
+        requirement_file = create_tempfile(self, ['foo'])
+        requirement_file_nested1 = create_tempfile(
+            self, ['bar\n-r {}'.format(requirement_file)])
+        requirement_file_nested2 = create_tempfile(
+            self, ['baz\n-r {}'.format(requirement_file_nested1)])
+        parsed = parsing.parse_reqfile(requirement_file_nested2)
+
+        self.assertDictEqual(parsed, {REPO_PYPI: [
+            get_req('baz'), get_req('bar'), get_req('foo')]})
